@@ -36,12 +36,16 @@
 #include <utility>      // for pair
 #include <vector>       // for vector
 
+#ifdef LIBSEMIGROUPS_VERBOSE
+#include <unordered_set>  // for unordered_set
+#endif
+
+#include "libsemigroups/config.hpp"        // for LIBSEMIGROUPS_DEBUG
 #include "libsemigroups/constants.hpp"     // for POSITIVE_INFINITY
+#include "libsemigroups/debug.hpp"         // for LIBSEMIGROUPS_ASSERT
 #include "libsemigroups/knuth-bendix.hpp"  // for KnuthBendix, KnuthBendi...
-#include "libsemigroups/libsemigroups-config.hpp"  // for LIBSEMIGROUPS_DEBUG
-#include "libsemigroups/libsemigroups-debug.hpp"   // for LIBSEMIGROUPS_ASSERT
-#include "libsemigroups/order.hpp"                 // for shortlex_compare
-#include "libsemigroups/report.hpp"                // for REPORT
+#include "libsemigroups/order.hpp"         // for shortlex_compare
+#include "libsemigroups/report.hpp"        // for REPORT
 #include "libsemigroups/string.hpp"  // for detail::is_suffix, maximum_comm...
 #include "libsemigroups/timer.hpp"   // for detail::Timer
 #include "libsemigroups/types.hpp"   // for word_type
@@ -270,6 +274,7 @@ namespace libsemigroups {
             _confluence_known(false),
             _inactive_rules(),
             _internal_is_same_as_external(false),
+            _contains_empty_string(false),
             _kb(kb),
             _min_length_lhs_rule(std::numeric_limits<size_t>::max()),
             _overlap_measure(nullptr),
@@ -279,7 +284,7 @@ namespace libsemigroups {
             _total_rules(0) {
         _next_rule_it1 = _active_rules.end();  // null
         _next_rule_it2 = _active_rules.end();  // null
-        this->set_overlap_policy(policy::overlap::ABC);
+        this->set_overlap_policy(options::overlap::ABC);
 #ifdef LIBSEMIGROUPS_VERBOSE
         _max_stack_depth        = 0;
         _max_word_length        = 0;
@@ -320,20 +325,20 @@ namespace libsemigroups {
       }
 
       static internal_char_type uint_to_internal_char(size_t a) {
-        LIBSEMIGROUPS_ASSERT(a
-                             <= std::numeric_limits<internal_char_type>::max());
+        LIBSEMIGROUPS_ASSERT(
+            a <= size_t(std::numeric_limits<internal_char_type>::max()));
 #ifdef LIBSEMIGROUPS_DEBUG
-        LIBSEMIGROUPS_ASSERT(a <= std::numeric_limits<internal_char_type>::max()
-                                      - 97);
+        LIBSEMIGROUPS_ASSERT(
+            a <= size_t(std::numeric_limits<internal_char_type>::max() - 97));
         return static_cast<internal_char_type>(a + 97);
 #else
         return static_cast<internal_char_type>(a + 1);
 #endif
       }
 
-      static internal_string_type uint_to_internal_string(size_t const i) {
-        LIBSEMIGROUPS_ASSERT(i
-                             <= std::numeric_limits<internal_char_type>::max());
+      static internal_string_type uint_to_internal_string(size_t i) {
+        LIBSEMIGROUPS_ASSERT(
+            i <= size_t(std::numeric_limits<internal_char_type>::max()));
         return internal_string_type({uint_to_internal_char(i)});
       }
 
@@ -403,7 +408,7 @@ namespace libsemigroups {
         auto qq = new external_string_type(q);
         external_to_internal_string(*pp);
         external_to_internal_string(*qq);
-        add_rule(new_rule(pp, qq));
+        push_stack(new_rule(pp, qq));
       }
 
       void add_rules(KnuthBendixImpl const* impl) {
@@ -435,7 +440,7 @@ namespace libsemigroups {
         return rules;
       }
 
-      size_t nr_rules() const {
+      size_t number_of_rules() const {
         return _active_rules.size();
       }
 
@@ -498,13 +503,10 @@ namespace libsemigroups {
         _max_active_rules = std::max(_max_active_rules, _active_rules.size());
         _unique_lhs_rules.insert(*rule->lhs());
 #endif
-        if (!_set_rules.emplace(RuleLookup(rule)).second) {
-          // The rules are not reduced, this should only happen if we are
-          // calling add_rule from outside the class (i.e. we are initialising
-          // the KnuthBendix).
-          push_stack(rule);
-          return;  // Do not activate or actually add the rule at this point
-        }
+        LIBSEMIGROUPS_ASSERT(_set_rules.emplace(RuleLookup(rule)).second);
+#ifndef LIBSEMIGROUPS_DEBUG
+        _set_rules.emplace(RuleLookup(rule));
+#endif
         rule->activate();
         _active_rules.push_back(rule);
         if (_next_rule_it1 == _active_rules.end()) {
@@ -519,6 +521,9 @@ namespace libsemigroups {
           // orderings (such as RECURSIVE)
           _min_length_lhs_rule = rule->lhs()->size();
         }
+        if (!_contains_empty_string) {
+          _contains_empty_string = rule->lhs()->empty() || rule->rhs()->empty();
+        }
         LIBSEMIGROUPS_ASSERT(_set_rules.size() == _active_rules.size());
       }
 
@@ -552,6 +557,10 @@ namespace libsemigroups {
       }
 
      public:
+      bool contains_empty_string() const noexcept {
+        return _contains_empty_string;
+      }
+
       //////////////////////////////////////////////////////////////////////////
       // KnuthBendixImpl - other methods - public
       //////////////////////////////////////////////////////////////////////////
@@ -581,20 +590,20 @@ namespace libsemigroups {
         return uu == vv;
       }
 
-      void set_overlap_policy(policy::overlap p) {
+      void set_overlap_policy(options::overlap p) {
         if (p == _kb->_settings._overlap_policy
             && _overlap_measure != nullptr) {
           return;
         }
         delete _overlap_measure;
         switch (p) {
-          case policy::overlap::ABC:
+          case options::overlap::ABC:
             _overlap_measure = new ABC();
             break;
-          case policy::overlap::AB_BC:
+          case options::overlap::AB_BC:
             _overlap_measure = new AB_BC();
             break;
-          case policy::overlap::MAX_AB_BC:
+          case options::overlap::MAX_AB_BC:
             _overlap_measure = new MAX_AB_BC();
             break;
           default:
@@ -770,6 +779,10 @@ namespace libsemigroups {
       // KnuthBendixImpl - main methods - public
       //////////////////////////////////////////////////////////////////////////
 
+      bool confluent_known() const noexcept {
+        return _confluence_known;
+      }
+
       bool confluent() const {
         if (!_stack.empty()) {
           return false;
@@ -828,9 +841,10 @@ namespace libsemigroups {
               }
             }
             if (_kb->report()) {
-              REPORT_DEFAULT("checked %d pairs of overlaps out of %d\n",
-                             seen,
-                             _active_rules.size() * _active_rules.size());
+              REPORT_DEFAULT("checked %llu pairs of overlaps out of %llu\n",
+                             uint64_t(seen),
+                             uint64_t(_active_rules.size())
+                                 * uint64_t(_active_rules.size()));
             }
           }
           if (_kb->running() && _kb->stopped()) {
@@ -996,6 +1010,7 @@ namespace libsemigroups {
       mutable std::atomic<bool>        _confluence_known;
       mutable std::list<Rule*>         _inactive_rules;
       bool                             _internal_is_same_as_external;
+      bool                             _contains_empty_string;
       KnuthBendix*                     _kb;
       size_t                           _min_length_lhs_rule;
       std::list<Rule const*>::iterator _next_rule_it1;
